/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>



/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup UART
  * @{
  */

/**
  * @addtogroup UART_DMA
  * @{
  */

/* Private typedef --------------------------------------------------------------*/

/* Private define ---------------------------------------------------------------*/
#define ARRAY_NUM(arr_nanme)       (uint32_t)(sizeof(arr_nanme) / sizeof(*(arr_nanme)))

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/
uint8_t rx_buf[10];
uint8_t tx_buf[] = "UART DMA receive and transmit example, please input 10 bytes:\r\n";

/* Private functions ------------------------------------------------------------*/

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();
  /* Configure the system tick */
  SystickInit();

  /* LED Init */
  EVAL_LedInit(LED1);

  /********************* Operation  *********************/
  /* UART0 Init */
  EVAL_ComInit();

  printf("UART DMA test example!\r\n");

  /* Config DMA */
  RCC_Unlock();
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_DMA, ENABLE);
  RCC_Lock();
  DMA_InitTypeStruct DMA_InitStruct;
  DMA_StructInit(&DMA_InitStruct);
  DMA_DeInit(DMA_CH0);
  /* Trasmit */
  DMA_InitStruct.DMA_CICRState = DISABLE;
  DMA_InitStruct.DMA_DataNum =  1 - 1;
  DMA_InitStruct.DMA_DataSize = DMA_DATA_SIZE_8;
  DMA_InitStruct.DMA_DstAddr = (uint32_t)&UART0->TDR;
  DMA_InitStruct.DMA_DstIncMode = DMA_ADDR_MODE_RESET;
  DMA_InitStruct.DMA_ErrIntState = DISABLE;
  DMA_InitStruct.DMA_FISIntState = DISABLE;
  DMA_InitStruct.DMA_ReqSource = DMA_UART0_TX_REQ;
  DMA_InitStruct.DMA_RoundNum = ARRAY_NUM(tx_buf) - 1;
  DMA_InitStruct.DMA_SrcAddr = (uint32_t)tx_buf;
  DMA_InitStruct.DMA_SrcIncMode = DMA_ADDR_MODE_INCREASE;
  DMA_Init(DMA_CH0, &DMA_InitStruct);

  DMA_Cmd(DMA_CH0, ENABLE);
  UART_DMACmd(UART0, UART_DMA_REQ_TX, ENABLE);

  /* wait DMA channel transfer complete */
  while(RESET == DMA_GetIntFlagStatus(DMA_CH0, DMA_INT_FLAG_FINISH))
  {
  }

  DMA_Cmd(DMA_CH0, DISABLE);
  UART_DMACmd(UART0, UART_DMA_REQ_TX, DISABLE);

  /* Receive */
  DMA_StructInit(&DMA_InitStruct);
  DMA_DeInit(DMA_CH1);
  DMA_InitStruct.DMA_CICRState = DISABLE;
  DMA_InitStruct.DMA_DataNum = 1 - 1;
  DMA_InitStruct.DMA_DataSize = DMA_DATA_SIZE_8;
  DMA_InitStruct.DMA_DstAddr = (uint32_t)rx_buf;
  DMA_InitStruct.DMA_DstIncMode = DMA_ADDR_MODE_INCREASE;
  DMA_InitStruct.DMA_ErrIntState = DISABLE;
  DMA_InitStruct.DMA_FISIntState = DISABLE;
  DMA_InitStruct.DMA_ReqSource = DMA_UART0_RX_REQ;
  DMA_InitStruct.DMA_RoundNum = 10 - 1;
  DMA_InitStruct.DMA_SrcAddr = (uint32_t)&UART0->RDR;
  DMA_InitStruct.DMA_SrcIncMode = DMA_ADDR_MODE_RESET;
  DMA_Init(DMA_CH1, &DMA_InitStruct);
  /* Enable DMA */
  DMA_Cmd(DMA_CH1, ENABLE);
  UART_DMACmd(UART0, UART_DMA_REQ_RX, ENABLE);

  /* wait DMA channel transfer complete */
  while(RESET == DMA_GetIntFlagStatus(DMA_CH1, DMA_INT_FLAG_FINISH))
  {
  }

  DMA_Cmd(DMA_CH1, DISABLE);

  UART_DMACmd(UART0, UART_DMA_REQ_RX, DISABLE);

  printf("UART DMA Receive: %s\r\n", rx_buf);

  uint32_t ledTime = 0;

  while(1)
  {
    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}

/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}


/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
